import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

abstract class BaseState<T extends StatefulWidget> extends State<StatefulWidget> {
  Center get centerCircularProgressIndicator {
    return Center(
      child: CircularProgressIndicator(),
    );
  }

  Divider get divider => Divider(
        indent: 12,
        height: 0.5,
        endIndent: 12,
      );

  showProgress({barrierDismissible: false}) {
    return showDialog(
        barrierDismissible: barrierDismissible,
        context: context,
        builder: (BuildContext context) {
          return centerCircularProgressIndicator;
        });
  }

  hideProgress() {
    return Navigator.pop(context);
  }

  /// Allows the user to call setState() within the Controller.
  void refresh() {
    if (mounted) {
      /// Refresh the interface by 'rebuilding' the Widget Tree
      setState(() {});
    } else {
      debugPrint("mounted");
    }
  }

  // user defined function
  void showAlert(
    String content, {
    String title,
    VoidCallback onPressed,
  }) {
    // flutter defined function
    showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: Text(title ?? ''),
          content: SingleChildScrollView(
            child: Text(content ?? ''),
          ),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            FlatButton(
              child: Text("OK"),
              onPressed: () {
                Navigator.of(context).pop();
                if (onPressed != null) {
                  onPressed();
                }
              },
            ),
          ],
        );
      },
    );
  }

  // user defined function
  void showConformAlert(
    String content, {
    String title,
    VoidCallback onPressed,
  }) {
    // flutter defined function
    showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: Text(title ?? ''),
          content: SingleChildScrollView(
            child: Text(content ?? ''),
          ),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            FlatButton(
              child: Text("取消"),
              onPressed: () {
                Navigator.of(context).pop();
              },
            ),
            FlatButton(
              child: Text("确认"),
              onPressed: () {
                Navigator.of(context).pop();
                if (onPressed != null) {
                  onPressed();
                }
              },
            ),
          ],
        );
      },
    );
  }

  showConformBottomSheet(String title, GestureTapCallback onTap) {
    return showModalBottomSheet(
        context: context,
        builder: (context) {
          return SafeArea(
              child: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              ListTile(
                title: Text(title, style: Theme.of(context).textTheme.button),
              ),
              ListTile(
                title: Text('确认', style: Theme.of(context).textTheme.button.copyWith(color: Colors.red)),
                onTap: () {
                  Navigator.pop(context);
                  onTap();
                },
              ),
              ListTile(
                title: Text('取消', style: Theme.of(context).textTheme.button),
                onTap: () {
                  Navigator.pop(context);
                },
              ),
            ],
          ));
        });
  }
}
