import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:max_flutter_engine/StringUtils.dart';
import 'package:max_flutter_engine/generated/l10n.dart';
import 'package:package_info/package_info.dart';
import 'package:permission_handler/permission_handler.dart';

typedef PermissionCheckCallback = void Function(bool);

class MPermissionHelper {
  static check(BuildContext context, Permission permission, PermissionCheckCallback callback) async {
    if (await permission.isUndetermined || await permission.isDenied) {
      // 从未请求过该权限
      requestPermission(callback, permission);
      return;
    }

    if (await permission.isGranted) {
      // 已经授权
      callback(true);
    } else {
      final packageInfo = await PackageInfo.fromPlatform();
      final content = ES.of(context).please_allow_assess_info(packageInfo.appName, permissionName(context, permission), permissionFor(context, permission),
          StringUtils.capitalize(permissionName(context, permission)), packageInfo.appName);

      // flutter defined function
      showDialog(
          context: context,
          builder: (BuildContext context) {
            // return object of type Dialog
            return AlertDialog(
              title: Text(ES.of(context).please_allow_assess),
              content: SingleChildScrollView(
                child: Text(content ?? ''),
              ),
              actions: <Widget>[
                // usually buttons at the bottom of the dialog
                FlatButton(
                  child: Text(ES.of(context).no_now),
                  onPressed: () {
                    Navigator.of(context).pop();
                    callback(false);
                  },
                ), // usually buttons at the bottom of the dialog
                FlatButton(
                  child: Text(ES.of(context).settings),
                  onPressed: () async {
                    Navigator.of(context).pop();
                    openAppSettings();
                    callback(false);
                  },
                ),
              ],
            );
          });
    }
  }

  static String permissionName(BuildContext context, Permission permission) {
    switch (permission) {
      case Permission.calendar:
        return ES.of(context).permission_calendar;
      case Permission.camera:
        return ES.of(context).permission_camera;
      case Permission.contacts:
        return ES.of(context).permission_contacts;
      // case Permission.location:
      //   return ES.of(context).permission_location;
      // case Permission.locationAlways:
      //   return ES.of(context).permission_locationalways;
      // case Permission.locationWhenInUse:
      //   return ES.of(context).permission_locationwheninuse;
      case Permission.mediaLibrary:
        return ES.of(context).permission_medialibrary;
      case Permission.microphone:
        return ES.of(context).permission_microphone;
      // case Permission.phone:
      // return ES.of(context).permission_phone;
      case Permission.photos:
        return ES.of(context).permission_photos;
      case Permission.reminders:
        return ES.of(context).permission_reminders;
      case Permission.sensors:
        return ES.of(context).permission_sensors;
      case Permission.sms:
        return ES.of(context).permission_sms;
      case Permission.speech:
        return ES.of(context).permission_speech;
      case Permission.storage:
        return ES.of(context).permission_storage;
      case Permission.ignoreBatteryOptimizations:
        return ES.of(context).permission_ignorebatteryoptimizations;
      case Permission.unknown:
        return ES.of(context).permission_unknown;
    }
    return ES.of(context).permission_unknown;
  }

  static permissionFor(BuildContext context, Permission permission) {
    switch (permission) {
      case Permission.calendar:
        return ES.of(context).permission_calendar_info;
      case Permission.camera:
        return ES.of(context).permission_camera_info;
      case Permission.contacts:
        return ES.of(context).permission_contacts_info;
      // case Permission.location:
      //   return ES.of(context).permission_location_info;
      // case Permission.locationAlways:
      //   return ES.of(context).permission_locationalways_info;
      // case Permission.locationWhenInUse:
      //   return ES.of(context).permission_locationwheninuse_info;
      case Permission.mediaLibrary:
        return ES.of(context).permission_medialibrary_info;
      case Permission.microphone:
        return ES.of(context).permission_microphone_info;
      // case Permission.phone:
      //   return ES.of(context).permission_phone_info;
      case Permission.photos:
        return ES.of(context).permission_photos_info;
      case Permission.reminders:
        return ES.of(context).permission_reminders_info;
      case Permission.sensors:
        return ES.of(context).permission_sensors_info;
      case Permission.sms:
        return ES.of(context).permission_sms_info;
      case Permission.speech:
        return ES.of(context).permission_speech_info;
      case Permission.storage:
        return ES.of(context).permission_storage_info;
      case Permission.ignoreBatteryOptimizations:
        return ES.of(context).permission_ignorebatteryoptimizations_info;
      case Permission.unknown:
        return ES.of(context).permission_unknown_info;
    }
    return ES.of(context).permission_unknown_info;
  }

  static requestPermission(PermissionCheckCallback callback, Permission permission) async {
    // You can request multiple permissions at once.
    Map<Permission, PermissionStatus> statuses = await [
      permission,
    ].request();
    callback(statuses[permission].isGranted);
  }

  static Future<Map<Permission, PermissionStatus>> requestPermissions(PermissionCheckCallback callback, List<Permission> permissions) async {
    // You can request multiple permissions at once.
    Map<Permission, PermissionStatus> statuses = await permissions.request();
    return statuses;
  }
}
