import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:max_flutter_engine/generated/l10n.dart';
import 'package:package_info/package_info.dart';
import 'package:max_flutter_engine/engine_config.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:url_launcher/url_launcher.dart';

typedef MRatingCheckerCallback = void Function(bool);

class MRatingHelper {
  // 单例公开访问点
  factory MRatingHelper() => _sharedInstance();

  // 静态私有成员，没有初始化
  static MRatingHelper _instance;

  // 私有构造函数
  MRatingHelper._() {
    // 具体初始化代码
  }

  // 静态、同步、私有访问点
  static MRatingHelper _sharedInstance() {
    if (_instance == null) {
      _instance = MRatingHelper._();
    }
    return _instance;
  }

  static const ratingDateKey = "rating_show_date";

  static Future<String> _getRatingSaveKey() async {
    PackageInfo packageInfo = await PackageInfo.fromPlatform();
    final ratingKey = "rating_${packageInfo.buildNumber}";
    return ratingKey;
  }

  static hasAskedToday() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String result = prefs.getString(ratingDateKey) == null ? "" : prefs.getString(ratingDateKey);
    var now = new DateTime.now();
    return "${now.year}-${now.month}-${now.day}" == result;
  }

  static setAsked() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var now = new DateTime.now();
    prefs.setString(ratingDateKey, "${now.year}-${now.month}-${now.day}");
  }

  static isRating() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String ratingKey = await _getRatingSaveKey();
    bool result = prefs.getBool(ratingKey) == null ? false : prefs.getBool(ratingKey);
    return result;
  }

  static rating() async {
    String ratingKey = await _getRatingSaveKey();
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setBool(ratingKey, true);
  }

  static chekStatus(BuildContext context, MRatingCheckerCallback callback) async {
    if (await hasAskedToday()) {
      callback(true);
      return;
    }
    if (await isRating()) {
      callback(true);
    } else {
      setAsked();
      PackageInfo packageInfo = await PackageInfo.fromPlatform();
      showDialog(
          context: context,
          builder: (_) => AlertDialog(
                  title: Text(ES.of(context).enjoying_appname(packageInfo.appName)),
                  content: Text(ES.of(context).tap_to_star_to_rate_it_on_the_app_store),
                  actions: <Widget>[
                    FlatButton(
                      child: Text(ES.of(context).not_now),
                      onPressed: () {
                        callback(false);
                        Navigator.of(context).pop();
                      },
                    ),
                    FlatButton(
                      child: Text(ES.of(context).go),
                      onPressed: () {
                        Navigator.of(context).pop();
                        jumpToRating(context);
                      },
                    )
                  ]));
    }
  }

  static jumpToRating(BuildContext context) async {
//    final result = await MaxFlutterEngine.requestReview;
//    if (result == true) {
//      MRatingHelper.rating();
//      return;
//    }
    // TODO: support different android market
    String url = Platform.isIOS ? EngineConfig.iosRateReviewURL : EngineConfig.googlePlayRateReviewURL;

    if (await canLaunch(url)) {
      await launch(url);
      MRatingHelper.rating();
    } else {
      final snackBar = SnackBar(
        content: Text(ES.of(context).tips_rating_jump_fail),
        action: SnackBarAction(
          label: ES.of(context).close,
          onPressed: () {},
        ),
      );
      Scaffold.of(context).showSnackBar(snackBar);
    }
  }
}
